@@ -1,4 +1,4 @@
-/*      $Id: acconfig.h,v 5.29 2004/09/11 19:56:51 lirc Exp $      */
+/*      $Id: acconfig.h,v 5.35 2005/12/26 09:51:57 lirc Exp $      */
 
 /*
  *  are you editing the correct file?
@@ -78,12 +78,24 @@
 /* Define if the ALSA library is installed */
 #undef HAVE_LIBALSA
 
+/* Define if the ALSA library with SB RC support is installed */
+#undef HAVE_ALSA_SB_RC
+
 /* Define if libusb is installed */
 #undef HAVE_LIBUSB
 
 /* Define if the complete vga libraries (vga, vgagl) are installed */
 #undef HAVE_LIBVGA
 
+/* defined if Linux input interface is available */
+#undef HAVE_LINUX_DEVINPUT
+
+/* defined if SCSI API is available */
+#undef HAVE_SCSI
+
+/* defined if soundcard API is available */
+#undef HAVE_SOUNDCARD
+
 /* define if you have vsyslog( prio, fmt, va_arg ) */
 #undef HAVE_VSYSLOG
 
@@ -97,13 +109,16 @@
 #define LIRC_MAJOR		61
 
 /* Set the IRQ for the lirc driver */
-#define LIRC_IRQ		4
+#undef LIRC_IRQ
 
 /* Set the port address for the lirc driver */
-#define LIRC_PORT		0x3f8
+#undef LIRC_PORT
 
 /* Set the timer for the parallel port driver */
-#define LIRC_TIMER		65536
+#undef LIRC_TIMER
+
+/* Define if you want to use lirc_it87 with an Asus Digimatrix */
+#undef LIRC_IT87_DIGIMATRIX
 
 /* Define if you have an animax serial port receiver */
 #undef LIRC_SERIAL_ANIMAX
@@ -123,6 +138,9 @@
 /* Define if you have an Igor Cesko receiver */
 #undef LIRC_SERIAL_IGOR
 
+/* Define if you have a Linksys NSLU2 and use CTS2+GreenLED */
+#undef LIRC_SERIAL_NSLU2
+
 /* Define if you want to cross-compile for the SA1100 */
 #undef LIRC_ON_SA1100
 
@@ -132,6 +150,9 @@
 /* Define if you want to use a Actisys Act200L */
 #undef LIRC_SIR_ACTISYS_ACT200L
 
+/* Define if you want to use a Actisys Act220L */
+#undef LIRC_SIR_ACTISYS_ACT220L
+
 /* Define if devfs support is present in current kernel */
 #undef LIRC_HAVE_DEVFS
 
