@@ -1,4 +1,4 @@
-/*      $Id: config_file.c,v 5.11 2002/07/01 18:54:01 lirc Exp $      */
+/*      $Id: config_file.c,v 5.20 2005/06/18 20:39:45 lirc Exp $      */
 
 /****************************************************************************
  ** config_file.c ***********************************************************
@@ -25,6 +25,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
+#include <ctype.h>
 
 #include "lircd.h"
 #include "ir_remote.h"
@@ -96,14 +97,14 @@ inline char *s_strdup(char * string)
         return (ptr);
 }
 
-inline ir_code s_strtocode(char *val)
+inline ir_code s_strtocode(const char *val)
 {
 	ir_code code=0;
 	char *endptr;
 
 	errno=0;
 #       ifdef LONG_IR_CODE
-	code=strtouq(val,&endptr,0);
+	code=strtoull(val,&endptr,0);
 	if((code==(unsigned long long) -1 && errno==ERANGE) ||
 	    strlen(endptr)!=0 || strlen(val)==0)
 	{
@@ -238,6 +239,7 @@ int addSignal(struct void_array *signals, char *val)
 	      
 struct ir_ncode *defineCode(char *key, char *val, struct ir_ncode *code)
 {
+	memset(code, 0, sizeof(*code));
         code->name=s_strdup(key);
         code->code=s_strtocode(val);
 #       ifdef LONG_IR_CODE
@@ -248,6 +250,35 @@ struct ir_ncode *defineCode(char *key, char *val, struct ir_ncode *code)
         return(code);
 }
 
+struct ir_code_node *defineNode(struct ir_ncode *code, const char *val)
+{
+	struct ir_code_node *node;
+	
+	node=s_malloc(sizeof(*node));
+	if(node==NULL) return NULL;
+	
+	node->code=s_strtocode(val);
+	node->next=NULL;
+	
+#       ifdef LONG_IR_CODE
+        LOGPRINTF(3,"                           0x%016llX", node->code);
+#       else
+        LOGPRINTF(3,"                           0x%016lX", node->code);
+#       endif
+
+	if(code->current==NULL)
+	{
+		code->next=node;
+		code->current=node;
+	}
+	else
+	{
+		code->current->next=node;
+		code->current=node;
+	}
+	return node;
+}
+
 int parseFlags(char *val)
 {
         struct flaglist *flaglptr;
@@ -270,6 +301,18 @@ int parseFlags(char *val)
 		flaglptr=all_flags;
 		while(flaglptr->name!=NULL){
 			if(strcasecmp(flaglptr->name,flag)==0){
+				if(flaglptr->flag&IR_PROTOCOL_MASK &&
+				   flags&IR_PROTOCOL_MASK)
+				{
+					logprintf(LOG_ERR, "error in "
+						  "configfile line %d:",
+						  line);
+					logprintf(LOG_ERR, "multiple "
+						  "protocols given in flags: "
+						  "\"%s\"",flag);
+					parse_error=1;
+					return(0);
+				}
 				flags=flags|flaglptr->flag;
 				LOGPRINTF(3,"flag %s recognized",
 					  flaglptr->name);
@@ -300,6 +343,16 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		LOGPRINTF(1,"parsing %s remote",val);
 		return(1);
 	}
+#ifdef DYNCODES
+	if ((strcasecmp("dyncodes_name",key))==0){
+		if(rem->dyncodes_name!=NULL)
+		{
+			free(rem->dyncodes_name);
+		}
+		rem->dyncodes_name=s_strdup(val);
+		return(1);
+	}
+#endif
 	else if ((strcasecmp("bits",key))==0){
 		rem->bits=s_strtoi(val);
 		return(1);
@@ -356,6 +409,10 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		rem->toggle_mask=s_strtocode(val);
 		return(1);
 	}
+	else if (strcasecmp("rc6_mask",key)==0){
+		rem->rc6_mask=s_strtocode(val);
+		return(1);
+	}
 	/* obsolete name */
 	else if (strcasecmp("repeat_bit",key)==0){
 		rem->toggle_bit=s_strtoi(val);
@@ -373,6 +430,48 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		rem->duty_cycle=s_strtoui(val);
 		return(1);
 	}
+	else if (strcasecmp("baud",key)==0){
+		rem->baud=s_strtoui(val);
+		return(1);
+	}
+	else if (strcasecmp("serial_mode",key)==0){
+		if(val[0]<'5' || val[0]>'9')
+		{
+			logprintf(LOG_ERR,"error in configfile line %d:",
+				  line);
+			logprintf(LOG_ERR,"bad bit count");
+			parse_error=1;
+			return 0;
+		}
+		rem->bits_in_byte=val[0]-'0';
+		switch(toupper(val[1]))
+		{
+		case 'N':
+			rem->parity = IR_PARITY_NONE;
+			break;
+		case 'E':
+			rem->parity = IR_PARITY_EVEN;
+			break;
+		case 'O':
+			rem->parity = IR_PARITY_ODD;
+			break;
+		default:
+			logprintf(LOG_ERR,"error in configfile line %d:",
+				  line);
+			logprintf(LOG_ERR,"unsupported parity mode");
+			parse_error=1;
+			return 0;
+		}
+		if(strcmp(val+2, "1.5")==0)
+		{
+			rem->stop_bits=3;
+		}
+		else
+		{
+			rem->stop_bits=s_strtoui(val+2)*2;
+		}
+		return(1);
+	}
 	else if (val2!=NULL)
 	{
 		if (strcasecmp("header",key)==0){
@@ -442,6 +541,7 @@ struct ir_remote * read_config(FILE *f)
         struct void_array codes_list,raw_codes,signals;
 	struct ir_ncode raw_code={NULL,0,0,NULL};
 	struct ir_ncode name_code={NULL,0,0,NULL};
+	struct ir_ncode *code;
 	int mode=ID_none;
 
 	line=0;
@@ -506,7 +606,7 @@ struct ir_remote * read_config(FILE *f)
 						parse_error=1;
 						break;
 					}
-					rem->flags|=RAW_CODES;
+					set_protocol(rem, RAW_CODES);
 					raw_code.code=0;
                                         init_void_array(&raw_codes,30, sizeof(struct ir_ncode));
                                         mode=ID_raw_codes;
@@ -527,13 +627,17 @@ struct ir_remote * read_config(FILE *f)
                                                 rem=rem->next;
                                         }
 				}else if(mode==ID_codes){
-					add_void_array(&codes_list, defineCode(key, val, &name_code));
-					if(!parse_error && val2!=NULL)
+					code=defineCode(key, val, &name_code);
+					while(!parse_error && val2!=NULL)
 					{
-						logprintf(LOG_WARNING,"garbage after '%s'"
-							  " code in line %d ignored",
-							  key,line);
+						struct ir_code_node *node;
+						
+						if(val2[0]=='#') break; /* comment */
+						node=defineNode(code, val2);
+						val2=strtok(NULL, " \t");
 					}
+					code->current=NULL;
+					add_void_array(&codes_list, code);
                                 }else{
                                         logprintf(LOG_ERR,"error in configfile line %d:",line);
 					logprintf(LOG_ERR,"unknown section \"%s\"",val);
@@ -565,7 +669,7 @@ struct ir_remote * read_config(FILE *f)
 						if(raw_code.length%2==0)
 						{
 							logprintf(LOG_ERR,"error in configfile line %d:",line);
-							logprintf(LOG_ERR,"bad signal length",val);
+							logprintf(LOG_ERR,"bad signal length");
 							parse_error=1;
 						}
 						if(!add_void_array(&raw_codes, &raw_code))
@@ -588,6 +692,14 @@ struct ir_remote * read_config(FILE *f)
                                                 break;
                                         }
 
+#                                       ifdef DYNCODES
+					if(rem->dyncodes_name==NULL)
+					{
+						rem->dyncodes_name=s_strdup("unknown");
+					}
+					rem->dyncodes[0].name=rem->dyncodes_name;
+					rem->dyncodes[1].name=rem->dyncodes_name;
+#                                       endif
 					/* not really necessary because we
 					   clear the alloced memory */
                                         rem->next=NULL;
@@ -599,13 +711,17 @@ struct ir_remote * read_config(FILE *f)
 						logprintf(LOG_WARNING,"repeat_gap will be ignored if CONST_LENGTH flag is set");
 					}
 				}else if(mode==ID_codes){
-					add_void_array(&codes_list, defineCode(key, val, &name_code));
-					if(!parse_error && val2!=NULL)
+					code=defineCode(key, val, &name_code);
+					while(!parse_error && val2!=NULL)
 					{
-						logprintf(LOG_WARNING,"garbage after '%s'"
-							  " code in line %d ignored",
-							  key,line);
+						struct ir_code_node *node;
+						
+						if(val2[0]=='#') break; /* comment */
+						node=defineNode(code, val2);
+						val2=strtok(NULL, " \t");
 					}
+					code->current=NULL;
+					add_void_array(&codes_list, code);
                                 }else{
                                         logprintf(LOG_ERR,"error in configfile line %d:",line);
 					logprintf(LOG_ERR,"unknown section %s",val);
@@ -630,13 +746,17 @@ struct ir_remote * read_config(FILE *f)
 					}
 					break;
 				case ID_codes:
-					add_void_array(&codes_list, defineCode(key, val, &name_code));
-					if(!parse_error && val2!=NULL)
+					code=defineCode(key, val, &name_code);
+					while(!parse_error && val2!=NULL)
 					{
-						logprintf(LOG_WARNING,"garbage after '%s'"
-							  " code in line %d ignored",
-							  key,line);
+						struct ir_code_node *node;
+						
+						if(val2[0]=='#') break; /* comment */
+						node=defineNode(code, val2);
+						val2=strtok(NULL, " \t");
 					}
+					code->current=NULL;
+					add_void_array(&codes_list, code);
 					break;
 				case ID_raw_codes:
 				case ID_raw_name:
@@ -649,7 +769,7 @@ struct ir_remote * read_config(FILE *f)
 							if(raw_code.length%2==0)
 							{
 								logprintf(LOG_ERR,"error in configfile line %d:",line);
-								logprintf(LOG_ERR,"bad signal length",val);
+								logprintf(LOG_ERR,"bad signal length");
 								parse_error=1;
 							}
 							if(!add_void_array(&raw_codes, &raw_code))
@@ -730,6 +850,8 @@ struct ir_remote * read_config(FILE *f)
                 return((void *) -1);
         }
 	/* kick reverse flag */
+	/* handle RC6 flag to be backwards compatible: previous RC-6
+	   config files did not set rc6_mask */
 	rem=top_rem;
 	while(rem!=NULL)
 	{
@@ -753,11 +875,41 @@ struct ir_remote * read_config(FILE *f)
 				codes->code=reverse(codes->code,rem->bits);
 				codes++;
 			}
-			/* rem->flags=rem->flags&(~REVERSE); */
+			 rem->flags=rem->flags&(~REVERSE);
+			 rem->flags=rem->flags|COMPAT_REVERSE;
 			/* don't delete the flag because we still need
 			   it to remain compatible with older versions
 			*/
 		}
+		if(rem->flags&RC6 && rem->rc6_mask==0 && rem->toggle_bit>0)
+		{
+			int all_bits=rem->pre_data_bits+
+				rem->bits+
+				rem->post_data_bits;
+			
+			rem->rc6_mask=((ir_code) 1)<<(all_bits-rem->toggle_bit);
+		}
+		if(is_serial(rem))
+		{
+			lirc_t base;
+			
+			if(rem->baud>0)
+			{
+				base=1000000/rem->baud;
+				if(rem->pzero==0 && rem->szero==0)
+				{
+					rem->pzero=base;
+				}
+				if(rem->pone==0 && rem->sone==0)
+				{
+					rem->sone=base;
+				}
+			}
+			if(rem->bits_in_byte==0)
+			{
+				rem->bits_in_byte=8;
+			}
+		}
 		rem=rem->next;
 	}
 
@@ -776,15 +928,27 @@ void free_config(struct ir_remote *remotes)
 	{
 		next=remotes->next;
 
+#               ifdef DYNCODES
+		if(remotes->dyncodes_name!=NULL) free(remotes->dyncodes_name);
+#               endif
 		if(remotes->name!=NULL) free(remotes->name);
 		if(remotes->codes!=NULL)
 		{
 			codes=remotes->codes;
 			while(codes->name!=NULL)
 			{
+				struct ir_code_node *node,*next_node;
+				
 				free(codes->name);
 				if(codes->signals!=NULL)
 					free(codes->signals);
+				node=codes->next;
+				while(node)
+				{
+					next_node=node->next;
+					free(node);
+					node=next_node;
+				}
 				codes++;
 			}
 			free(remotes->codes);
