@@ -1,4 +1,4 @@
-/*      $Id: config_file.h,v 5.3 2000/07/05 12:25:03 columbus Exp $      */
+/*      $Id: config_file.h,v 5.9 2005/04/23 12:08:46 lirc Exp $      */
 
 /****************************************************************************
  ** config_file.h ***********************************************************
@@ -25,20 +25,24 @@ struct flaglist {
 
 static struct flaglist all_flags[]=
 {
+        {"RAW_CODES",       RAW_CODES},
 	{"RC5",             RC5},
+	{"SHIFT_ENC",       SHIFT_ENC}, /* obsolete */
 	{"RC6",             RC6},
 	{"RCMM",            RCMM},
-	{"SHIFT_ENC",       SHIFT_ENC}, /* obsolete */
 	{"SPACE_ENC",       SPACE_ENC},
+	{"SPACE_FIRST",     SPACE_FIRST},
+        {"GOLDSTAR",        GOLDSTAR},
+	{"GRUNDIG",         GRUNDIG},
+	{"SERIAL",          SERIAL},
+	
 	{"REVERSE",         REVERSE},
 	{"NO_HEAD_REP",     NO_HEAD_REP},
         {"NO_FOOT_REP",     NO_FOOT_REP},
 	{"CONST_LENGTH",    CONST_LENGTH}, /* remember to adapt warning
 					      message when changing this */
-        {"RAW_CODES",       RAW_CODES},
         {"REPEAT_HEADER",   REPEAT_HEADER},
-	
-        {NULL,0},
+	{NULL,0},
 };
 
 /*
@@ -69,7 +73,7 @@ inline void * get_void_array(struct void_array *ar);
 /* some safer functions */
 void * s_malloc(size_t size);
 char * s_strdup(char * string);
-ir_code s_strtocode(char *val);
+ir_code s_strtocode(const char *val);
 unsigned long  s_strtoul(char *val);
 int s_strtoi(char *val);
 unsigned int s_strtoui(char *val);
@@ -79,6 +83,7 @@ int checkMode(int is_mode, int c_mode, char *error);
 int parseFlags(char *val);
 int addSignal(struct void_array *signals, char *val);
 struct ir_ncode * defineCode(char *key, char *val, struct ir_ncode *code);
+struct ir_code_node *defineNode(struct ir_ncode *code, const char *val);
 int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem);
 struct ir_remote *read_config(FILE *f);
 void free_config(struct ir_remote *remotes);
