@@ -1,4 +1,4 @@
-/*      $Id: hardware.h,v 5.5 2001/12/08 15:07:03 lirc Exp $      */
+/*      $Id: hardware.h,v 5.9 2005/07/10 08:34:06 lirc Exp $      */
 
 /****************************************************************************
  ** hardware.h **************************************************************
@@ -14,7 +14,7 @@
 #define _HARDWARE_H
 
 #include "drivers/lirc.h"
-#include "ir_remote.h"
+#include "ir_remote_types.h"
 
 struct hardware
 {
@@ -32,11 +32,12 @@ struct hardware
 	int (*decode_func)(struct ir_remote *remote,
 			   ir_code *prep,ir_code *codep,ir_code *postp,
 			   int *repeat_flag,lirc_t *remaining_gapp);
-	lirc_t (*readdata)(void);
+	int (*ioctl_func)(unsigned int cmd, unsigned long arg);
+	lirc_t (*readdata)(lirc_t timeout);
 	char *name;
+	
+	int resolution;
 };
 
-#ifndef LIRC_NETWORK_ONLY
 extern struct hardware hw;
 #endif
-#endif
