@@ -6,19 +6,25 @@
 #include "hardware.h"
 #include "hw-types.h"
 /* Hardware types */
+extern struct hardware hw_alsa_usb;
 extern struct hardware hw_atilibusb;
 extern struct hardware hw_audio;
 extern struct hardware hw_audio_alsa;
 extern struct hardware hw_bte;
+extern struct hardware hw_bw6130;
 extern struct hardware hw_creative;
 extern struct hardware hw_creative_infracd;
 extern struct hardware hw_default;
 extern struct hardware hw_devinput;
 extern struct hardware hw_dsp;
+extern struct hardware hw_dvico;
+extern struct hardware hw_ea65;
 extern struct hardware hw_irman;
 extern struct hardware hw_livedrive_midi;
 extern struct hardware hw_livedrive_seq;
 extern struct hardware hw_logitech;
+extern struct hardware hw_mouseremote;
+extern struct hardware hw_mouseremote_ps2;
 extern struct hardware hw_mp3anywhere;
 extern struct hardware hw_pcmak;
 extern struct hardware hw_pinsys;
@@ -50,6 +56,7 @@ struct hardware hw_null=
 	NULL,               /* send_func */
 	NULL,               /* rec_func */
 	NULL,               /* decode_func */
+	NULL,               /* ioctl_func */
 	NULL,		    /* readdata */
 	"null",		    /* name */
 };
@@ -57,6 +64,9 @@ struct hardware hw_null=
 struct hardware *hw_list[] =
 {
 #ifdef LIRC_DRIVER_ANY
+#ifdef HAVE_ALSA_SB_RC
+	&hw_alsa_usb,
+#endif
 #ifdef HAVE_LIBUSB
 	&hw_atilibusb,
 #endif
@@ -67,11 +77,20 @@ struct hardware *hw_list[] =
 	&hw_audio_alsa,
 #endif
 	&hw_bte,
+	&hw_bw6130,
 	&hw_creative,
+#ifdef HAVE_SCSI
 	&hw_creative_infracd,
+#endif
 	&hw_default,
+#ifdef HAVE_LINUX_DEVINPUT
 	&hw_devinput,
+#endif
+#ifdef HAVE_SOUNDCARD
 	&hw_dsp,
+#endif
+	&hw_dvico,
+	&hw_ea65,
 #ifdef HAVE_LIBIRMAN
 	&hw_irman,
 #endif
@@ -79,6 +98,8 @@ struct hardware *hw_list[] =
 	&hw_livedrive_seq,
 	&hw_logitech,
 	&hw_mp3anywhere,
+	&hw_mouseremote,
+	&hw_mouseremote_ps2,
 	&hw_null,
 	&hw_pcmak,
 	&hw_pinsys,
@@ -103,7 +124,6 @@ struct hardware hw;
 int hw_choose_driver (char *name)
 {
 	int i;
-	char *device = hw.device;
 	
 	if(name==NULL){
 		hw = HW_DEFAULT;
@@ -116,10 +136,6 @@ int hw_choose_driver (char *name)
 		return -1;
 	hw = *hw_list[i];
 
-	/* just in case the device was already selected by the user */
-	if(device)
-		hw.device = device;
-
 	return 0;
 } 
 
