@@ -1,4 +1,4 @@
-/*      $Id: hw_default.c,v 5.27 2003/11/12 11:52:14 lirc Exp $      */
+/*      $Id: hw_default.c,v 5.31 2005/07/10 08:34:11 lirc Exp $      */
 
 /****************************************************************************
  ** hw_default.c ************************************************************
@@ -76,11 +76,12 @@ struct hardware hw_default=
 	default_send,       /* send_func */
 	default_rec,        /* rec_func */
 	default_decode,     /* decode_func */
+	default_ioctl,      /* ioctl_func */
 	default_readdata,
 	"default"
 };
 
-unsigned int min_freq=0,max_freq=0;
+static unsigned int min_freq=0,max_freq=0;
 
 /**********************************************************************
  *
@@ -89,6 +90,7 @@ unsigned int min_freq=0,max_freq=0;
  **********************************************************************/
 
 static int default_config_frequency();
+static int write_send_buffer(int lirc);
 
 /**********************************************************************
  *
@@ -331,7 +333,18 @@ int default_init()
 				  "driver is not yet supported by lircd");
 		}
 	}
-	if(hw.rec_mode==LIRC_MODE_CODE)
+	if(hw.rec_mode==LIRC_MODE_MODE2)
+	{
+		/* get resolution */
+		hw.resolution=0;
+		if(ioctl(hw.fd, LIRC_GET_REC_RESOLUTION, &hw.resolution)!=-1)
+		{
+			LOGPRINTF(1, "resolution of receiver: %d",
+				  hw.resolution);
+		}
+		
+	}
+	else if(hw.rec_mode==LIRC_MODE_CODE)
 	{
 		hw.code_length=8;
 	}
@@ -359,7 +372,7 @@ int default_init()
 	}
 	if(min_freq!=0 && max_freq!=0)
 	{
-		(void) default_config_frequency(min_freq,max_freq);
+		(void) default_config_frequency();
 	}
 #endif
 	return(1);
@@ -384,22 +397,11 @@ int default_deinit(void)
 	return(1);
 }
 
-int write_send_buffer(int lirc,int length,lirc_t *signals)
+static int write_send_buffer(int lirc)
 {
 #if defined(SIM_SEND) && !defined(DAEMONIZE)
 	int i;
 
-	if(send_buffer.wptr==0 && length>0 && signals!=NULL)
-	{
-		for(i=0;;)
-		{
-			printf("pulse %lu\n",(unsigned long) signals[i++]);
-			if(i>=length) break;
-			printf("space %lu\n",(unsigned long) signals[i++]);
-		}
-		return(length*sizeof(lirc_t));
-	}
-	
 	if(send_buffer.wptr==0) 
 	{
 		LOGPRINTF(1,"nothing to send");
@@ -413,11 +415,6 @@ int write_send_buffer(int lirc,int length,lirc_t *signals)
 	}
 	return(send_buffer.wptr*sizeof(lirc_t));
 #else
-	if(send_buffer.wptr==0 && length>0 && signals!=NULL)
-	{
-		return(write(lirc,signals,length*sizeof(lirc_t)));
-	}
-	
 	if(send_buffer.wptr==0) 
 	{
 		LOGPRINTF(1,"nothing to send");
@@ -486,7 +483,7 @@ int default_send(struct ir_remote *remote,struct ir_ncode *code)
 	}
 #endif
 
-	if(write_send_buffer(hw.fd,code->length,code->signals)==-1)
+	if(write_send_buffer(hw.fd)==-1)
 	{
 		logprintf(LOG_ERR,"write failed");
 		logperror(LOG_ERR,NULL);
@@ -497,16 +494,6 @@ int default_send(struct ir_remote *remote,struct ir_ncode *code)
 		gettimeofday(&remote->last_send,NULL);
 		remote->last_code=code;
 #if defined(SIM_SEND) && !defined(DAEMONIZE)
-		if(is_raw(remote) && is_const(remote))
-		{
-			int i;
-			
-			for(i=0;remote->remaining_gap>code->signals[i] &&
-				    i<code->length;i++)
-			{
-				remote->remaining_gap-=code->signals[i];
-			}
-		}
 		printf("space %lu\n",(unsigned long) remote->remaining_gap);
 #endif
 	}
@@ -592,3 +579,8 @@ static int default_config_frequency()
 	}
 	return(1);
 }
+
+int default_ioctl(unsigned int cmd, unsigned long arg)
+{
+	return ioctl(hw.fd, cmd, &arg);
+}
