@@ -1,4 +1,4 @@
-/*      $Id: hw_default.h,v 5.7 2002/05/04 09:36:27 lirc Exp $      */
+/*      $Id: hw_default.h,v 5.9 2005/07/10 08:34:11 lirc Exp $      */
 
 /****************************************************************************
  ** hw_default.h ************************************************************
@@ -21,11 +21,12 @@ inline lirc_t time_left(struct timeval *current,struct timeval *last,
 int default_init(void);
 int default_config(struct ir_remote *remotes);
 int default_deinit(void);
-int write_send_buffer(int lirc,int length,lirc_t *signals);
 int default_send(struct ir_remote *remote,struct ir_ncode *code);
 char *default_rec(struct ir_remote *remotes);
 int default_decode(struct ir_remote *remote,
 		   ir_code *prep,ir_code *codep,ir_code *postp,
 		   int *repeat_flagp,lirc_t *remaining_gapp);
+int default_ioctl(unsigned int cmd, unsigned long arg);
 lirc_t default_readdata (lirc_t timeout);
+
 #endif
