@@ -55,6 +55,7 @@ struct hardware hw_devinput=
 	NULL,			/* send_func */
 	devinput_rec,		/* rec_func */
 	devinput_decode,	/* decode_func */
+	NULL,                   /* ioctl_func */
 	NULL,			/* readdata */
 	"dev/input"
 };
@@ -72,7 +73,7 @@ int devinput_init()
 	}
 	
 #ifdef EVIOCGRAB
-	if (ioctl(hw.fd, EVIOCGRAB) == -1)
+	if (ioctl(hw.fd, EVIOCGRAB, 1) == -1)
 	{
 		logprintf(LOG_WARNING, "can't get exclusive access to events "
 			  "comming from `%s' interface",
