@@ -1,4 +1,4 @@
-/*      $Id: ir_remote.c,v 5.19 2002/02/22 14:47:47 lirc Exp $      */
+/*      $Id: ir_remote.c,v 5.24 2005/10/25 19:05:35 lirc Exp $      */
 
 /****************************************************************************
  ** ir_remote.c *************************************************************
@@ -109,7 +109,7 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 			  int *repeat_statep)
 {
 	ir_code pre_mask,code_mask,post_mask;
-	int repeat_state;
+	int repeat_state, found_code;
 	struct ir_ncode *codes,*found;
 	
 	pre_mask=code_mask=post_mask=0;
@@ -172,7 +172,7 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 			}
 			if(bit==remote->post_data_bits+remote->bits)
 			{
-				affected=&post;
+				affected=&pre;
 				current_bit=0;
 			}
 			mask_bit=mask&1;
@@ -210,35 +210,81 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 		LOGPRINTF(1,"post");
 	}
 	found=NULL;
+	found_code=0;
 	codes=remote->codes;
 	if(codes!=NULL)
 	{
 		while(codes->name!=NULL)
 		{
-			if((codes->code|code_mask)==(code|code_mask))
+			ir_code next_code;
+			
+			if(codes->next!=NULL && codes->current!=NULL)
 			{
-				found=codes;
-				if(has_toggle_mask(remote))
+				next_code=codes->current->code;
+			}
+			else
+			{
+				next_code=codes->code;
+			}
+			if((next_code|code_mask)==(code|code_mask))
+			{
+				found_code=1;
+				if(codes->next!=NULL)
 				{
-					if(!(remote->toggle_mask_state%2))
+					if(codes->current==NULL)
 					{
-						remote->toggle_code=codes;
-						LOGPRINTF(1,
-							  "toggle_mask_start");
-						break;
+						codes->current=codes->next;
 					}
-					if(codes!=remote->toggle_code)
+					else
 					{
-						remote->toggle_code=NULL;
-						return(NULL);
+						codes->current=
+							codes->next->next;
 					}
-					remote->toggle_code=NULL;
 				}
-				break;
+				found=codes;
+				if(codes->current==NULL)
+				{
+					break;
+				}
+			}
+			else
+			{
+				codes->current=NULL;
 			}
 			codes++;
 		}
 	}
+#       ifdef DYNCODES
+	if(!found_code)
+	{
+		if((remote->dyncodes[remote->dyncode].code|code_mask)!=
+		   (code|code_mask))
+		{
+			remote->dyncode++;
+			remote->dyncode%=2;
+		}
+		remote->dyncodes[remote->dyncode].code=code&(~code_mask);
+		found=&(remote->dyncodes[remote->dyncode]);
+		found_code=1;
+	}
+#       endif
+	if(found_code && found!=NULL && has_toggle_mask(remote))
+	{
+		if(!(remote->toggle_mask_state%2))
+		{
+			remote->toggle_code=found;
+			LOGPRINTF(1,"toggle_mask_start");
+		}
+		else
+		{
+			if(found!=remote->toggle_code)
+			{
+				remote->toggle_code=NULL;
+				return(NULL);
+			}
+			remote->toggle_code=NULL;
+		}
+	}
 	*repeat_statep=repeat_state;
 	return(found);
 }
@@ -253,7 +299,10 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 	LOGPRINTF(1,"found: %s",found->name);
 
 	gettimeofday(&current,NULL);
-	if(remote==last_remote && found==remote->last_code && repeat_flag &&
+	if(remote==last_remote &&
+	   ((found->next==NULL && found==remote->last_code) || 
+	    (found->next!=NULL)) &&
+	   repeat_flag &&
 	   time_elapsed(&remote->last_send,&current)<1000000 &&
 	   (!(remote->toggle_bit>0) || repeat_state==remote->repeat_state))
 	{
@@ -266,15 +315,21 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 				remote->toggle_mask_state=2;
 			}
 		}
-		else
+		else if(found->current==NULL)
 		{
-			remote->reps++;		
+			remote->reps++;
 		}
 	}
 	else
 	{
-		remote->reps=0;		
-		remote->last_code=found;
+		if(found->next!=NULL && found->current==NULL)
+		{
+			remote->reps=1;
+		}
+		else
+		{
+			remote->reps=0;
+		}
 		if(has_toggle_mask(remote))
 		{
 			remote->toggle_mask_state=1;
@@ -286,6 +341,7 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 		}
 	}
 	last_remote=remote;
+	remote->last_code=found;
 	remote->last_send=current;
 	remote->remaining_gap=remaining_gap;
 	
@@ -301,8 +357,12 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 		code=code<<remote->post_data_bits;
 		code|=remote->post_data;
 	}
-	if(remote->flags&REVERSE)
+	if(remote->flags&COMPAT_REVERSE)
 	{
+		/* actually this is wrong: pre, code and post should
+		   be rotated separately but we have to stay
+		   compatible with older software
+		 */
 		code=reverse(code,remote->pre_data_bits+
 			     remote->bits+remote->post_data_bits);
 	}
@@ -332,8 +392,9 @@ char *decode_all(struct ir_remote *remotes)
 
 			code=set_code(remote,ncode,repeat_state,repeat_flag,
 				      remaining_gap);
-			if(has_toggle_mask(remote) &&
-			   remote->toggle_mask_state%2)
+			if((has_toggle_mask(remote) &&
+			    remote->toggle_mask_state%2) ||
+			   ncode->current!=NULL)
 			{
 				decoding=NULL;
 				return(NULL);
@@ -347,18 +408,18 @@ char *decode_all(struct ir_remote *remotes)
 				     (code>>32),
 				     (unsigned long)
 				     (code&0xFFFFFFFF),
-				     remote->reps,
+				     remote->reps-(ncode->next ? 1:0),
 				     remote->last_code->name,
 				     remote->name);
 #else
-			len=snprintf(message,PACKET_SIZE,"%016llx %02x %s %s\n",
+			len=snprintf(message,PACKET_SIZE+1,"%016llx %02x %s %s\n",
 				     code,
-				     remote->reps,
+				     remote->reps-(ncode->next ? 1:0),
 				     remote->last_code->name,
 				     remote->name);
 #endif
 			decoding=NULL;
-			if(len==PACKET_SIZE+1)
+			if(len>=PACKET_SIZE+1)
 			{
 				logprintf(LOG_ERR,"message buffer overflow");
 				return(NULL);
