@@ -1,4 +1,4 @@
-/*      $Id: ir_remote.h,v 5.19 2001/12/08 15:07:03 lirc Exp $      */
+/*      $Id: ir_remote.h,v 5.32 2005/04/23 12:08:46 lirc Exp $      */
 
 /****************************************************************************
  ** ir_remote.h *************************************************************
@@ -11,8 +11,8 @@
  *
  */ 
 
-#ifndef _IR_REMOTE_H
-#define _IR_REMOTE_H
+#ifndef IR_REMOTE_H
+#define IR_REMOTE_H
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -22,112 +22,11 @@
 #include <stdlib.h>
 
 #include "drivers/lirc.h"
+#include "hardware.h"
 
-struct hardware;
+#include "ir_remote_types.h"
 
-#ifdef LONG_IR_CODE
-typedef unsigned long long ir_code;
-#else
-typedef unsigned long ir_code;
-#endif
-
-/*
-  Code with name string
-*/
-
-struct ir_ncode {
-	char *name;
-	ir_code code;
-        int length;
-        lirc_t *signals;
-};
-
-/*
-  struct ir_remote
-  defines the encoding of a remote control 
-*/
-
-/* definitions for flags */
-
-/* Don't forget to take a look at config_file.h when adding new flags */
-
-#define RC5             0x0001    /* IR data follows RC5 protocol */
-
-/* Hm, RC6 protocols seem to have changed the biphase semantics so
-   that lircd will calculate the bit-wise complement of the codes. But
-   this is only a guess as I did not have a datasheet... */
-
-#define RC6             0x0002    /* IR data follows RC6 protocol */
-#define RCMM            0x0004    /* IR data follows RC-MM protocol */
-#define SPACE_ENC	0x0008	  /* IR data is space encoded */
-#define REVERSE		0x0010
-#define NO_HEAD_REP	0x0020	  /* no header for key repeats */
-#define NO_FOOT_REP	0x0040	  /* no foot for key repeats */
-#define CONST_LENGTH    0x0080    /* signal length+gap is always constant */
-#define RAW_CODES       0x0100    /* for internal use only */
-#define REPEAT_HEADER   0x0200    /* header is also sent before repeat code */
-
-#define SHIFT_ENC	   RC5    /* IR data is shift encoded (name obsolete) */
-
-/* stop repeating after 600 signals (approx. 1 minute) */
-#define REPEAT_MAX 600
-
-#define DEFAULT_FREQ 38000
-
-struct ir_remote 
-{
-	char *name;                 /* name of remote control */
-	struct ir_ncode *codes;
-	int bits;                   /* bits (length of code) */
-	int flags;                  /* flags */
-	int eps;                    /* eps (_relative_ tolerance) */
-	int aeps;                   /* detecing _very short_ pulses is
-				       difficult with relative tolerance
-				       for some remotes,
-				       this is an _absolute_ tolerance
-				       to solve this problem
-				       usually you can say 0 here */
-	
-	/* pulse and space lengths of: */
-	
-	lirc_t phead,shead;         /* header */
-	lirc_t pthree,sthree;       /* 3 (only used for RC-MM) */
-	lirc_t ptwo,stwo;           /* 2 (only used for RC-MM) */
-	lirc_t pone,sone;           /* 1 */
-	lirc_t pzero,szero;         /* 0 */
-	lirc_t plead;		    /* leading pulse */
-	lirc_t ptrail;              /* trailing pulse */
-	lirc_t pfoot,sfoot;         /* foot */
-	lirc_t prepeat,srepeat;	    /* indicate repeating */
-
-	int pre_data_bits;          /* length of pre_data */
-	ir_code pre_data;           /* data which the remote sends before
-				       actual keycode */
-	int post_data_bits;         /* length of post_data */
-	ir_code post_data;          /* data which the remote sends after
-				       actual keycode */
-	lirc_t pre_p,pre_s;         /* signal between pre_data and keycode */
-	lirc_t post_p, post_s;      /* signal between keycode and post_code */
-
-	lirc_t gap;                 /* time between signals in usecs */
-	lirc_t repeat_gap;          /* time between two repeat codes
-				       if different from gap */
-	int toggle_bit;             /* 1..bits */
-	int min_repeat;             /* code is repeated at least x times
-				       code sent once -> min_repeat=0 */
-	unsigned int freq;          /* modulation frequency */
-	unsigned int duty_cycle;    /* 0<duty cycle<=100 */
-	
-	/* end of user editable values */
-	
-        int repeat_state;
-	int repeat_countdown;
-	struct ir_ncode *last_code; /* code received or sent last */
-	int reps;
-	struct timeval last_send;   /* time last_code was received or sent */
-	lirc_t remaining_gap;       /* remember gap for CONST_LENGTH remotes */
-        struct ir_remote *next;
-};
+extern struct hardware hw;
 
 static inline ir_code reverse(ir_code data,int bits)
 {
@@ -159,33 +58,70 @@ static inline int has_repeat(struct ir_remote *remote)
 	else return(0);
 }
 
+static inline void set_protocol(struct ir_remote *remote, int protocol)
+{
+	remote->flags&=~(IR_PROTOCOL_MASK);
+	remote->flags|=protocol;
+}
+
 static inline int is_raw(struct ir_remote *remote)
 {
-	if(remote->flags&RAW_CODES) return(1);
+	if((remote->flags&IR_PROTOCOL_MASK) == RAW_CODES) return(1);
 	else return(0);
 }
 
-static inline int is_biphase(struct ir_remote *remote)
+static inline int is_space_enc(struct ir_remote *remote)
 {
-	if(remote->flags&RC5 || remote->flags&RC6) return(1);
+	if((remote->flags&IR_PROTOCOL_MASK) == SPACE_ENC) return(1);
+	else return(0);
+}
+
+static inline int is_space_first(struct ir_remote *remote)
+{
+	if((remote->flags&IR_PROTOCOL_MASK) == SPACE_FIRST) return(1);
 	else return(0);
 }
 
 static inline int is_rc5(struct ir_remote *remote)
 {
-	if(remote->flags&RC5) return(1);
+	if((remote->flags&IR_PROTOCOL_MASK) == RC5) return(1);
 	else return(0);
 }
 
 static inline int is_rc6(struct ir_remote *remote)
 {
-	if(remote->flags&RC6) return(1);
+	if((remote->flags&IR_PROTOCOL_MASK) == RC6 ||
+	   remote->rc6_mask) return(1);
+	else return(0);
+}
+
+static inline int is_biphase(struct ir_remote *remote)
+{
+	if(is_rc5(remote) || is_rc6(remote)) return(1);
 	else return(0);
 }
 
 static inline int is_rcmm(struct ir_remote *remote)
 {
-	if(remote->flags&RCMM) return(1);
+	if((remote->flags&IR_PROTOCOL_MASK) == RCMM) return(1);
+	else return(0);
+}
+
+static inline int is_goldstar(struct ir_remote *remote)
+{
+	if((remote->flags&IR_PROTOCOL_MASK) == GOLDSTAR) return(1);
+	else return(0);
+}
+
+static inline int is_grundig(struct ir_remote *remote)
+{
+	if((remote->flags&IR_PROTOCOL_MASK) == GRUNDIG) return(1);
+	else return(0);
+}
+
+static inline int is_serial(struct ir_remote *remote)
+{
+	if((remote->flags&IR_PROTOCOL_MASK) == SERIAL) return(1);
 	else return(0);
 }
 
@@ -225,16 +161,50 @@ static inline int has_foot(struct ir_remote *remote)
 	else return(0);
 }
 
+static inline int has_toggle_mask(struct ir_remote *remote)
+{
+	if(remote->toggle_mask>0) return(1);
+	else return(0);
+}
+
 /* check if delta is inside exdelta +/- exdelta*eps/100 */
 
 static inline int expect(struct ir_remote *remote,lirc_t delta,lirc_t exdelta)
 {
+	int aeps = hw.resolution>remote->aeps ? hw.resolution:remote->aeps;
+	
 	if(abs(exdelta-delta)<exdelta*remote->eps/100 ||
-	   abs(exdelta-delta)<remote->aeps)
+	   abs(exdelta-delta)<aeps)
 		return 1;
 	return 0;
 }
 
+static inline int expect_at_least(struct ir_remote *remote,
+				  lirc_t delta, lirc_t exdelta)
+{
+	int aeps = hw.resolution>remote->aeps ? hw.resolution:remote->aeps;
+	
+	if(delta+exdelta*remote->eps/100>=exdelta ||
+	   delta+aeps>=exdelta)
+	{
+		return 1;
+	}
+	return 0;
+}
+
+static inline int expect_at_most(struct ir_remote *remote,
+				 lirc_t delta, lirc_t exdelta)
+{
+	int aeps = hw.resolution>remote->aeps ? hw.resolution:remote->aeps;
+	
+	if(delta<=exdelta+exdelta*remote->eps/100 ||
+	   delta<=exdelta+aeps)
+	{
+		return 1;
+	}
+	return 0;
+}
+
 /* only works if last <= current */
 static inline unsigned long time_elapsed(struct timeval *last,
 					 struct timeval *current)
