@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.46 2002/09/21 15:25:28 lirc Exp $      */
+/*      $Id: lircd.c,v 5.61 2005/10/30 11:11:46 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -57,14 +57,14 @@
 #include <sys/file.h>
 
 #ifndef timersub
-#define timersub(a, b, result)                                                \
-  do {                                                                        \
-    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                             \
-    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                          \
-    if ((result)->tv_usec < 0) {                                              \
-      --(result)->tv_sec;                                                     \
-      (result)->tv_usec += 1000000;                                           \
-    }                                                                         \
+#define timersub(a, b, result)                                            \
+  do {                                                                    \
+    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                         \
+    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                      \
+    if ((result)->tv_usec < 0) {                                          \
+      --(result)->tv_sec;                                                 \
+      (result)->tv_usec += 1000000;                                       \
+    }                                                                     \
   } while (0)
 #endif
 
@@ -87,10 +87,16 @@ static char *repeat_message=NULL;
 
 extern struct hardware hw;
 
-char *progname="lircd " VERSION;
+char *progname="lircd";
 char *configfile=LIRCDCFGFILE;
+#ifndef USE_SYSLOG
 char *logfile=LOGFILE;
-FILE *pidfile;
+#else
+static const char *syslogident = "lircd-" VERSION;
+#endif
+FILE *pidf;
+char *pidfile = PIDFILE;
+char *lircdfile = LIRCD;
 
 struct protocol_directive directives[] =
 {
@@ -99,6 +105,7 @@ struct protocol_directive directives[] =
 	{"SEND_START",send_start},
 	{"SEND_STOP",send_stop},
 	{"VERSION",version},
+	{"SET_TRANSMITTERS",set_transmitters},
 	{NULL,NULL}
 	/*
 	{"DEBUG",debug},
@@ -156,6 +163,9 @@ int daemonized=0;
 static sig_atomic_t term=0,hup=0,alrm=0;
 static int termsig;
 
+/* set_transmitters only supports 32 bit int */
+#define MAX_TX (CHAR_BIT*sizeof(unsigned long))
+
 inline int max(int a,int b)
 {
 	return(a>b ? a:b);
@@ -192,7 +202,14 @@ inline int write_socket(int fd, char *buf, int len)
 
 	while(todo)
 	{
+#ifdef SIM_REC
+		do{
+			done=write(fd,buf,todo);
+		}
+		while(done<0 && errno == EAGAIN);
+#else
 		done=write(fd,buf,todo);
+#endif
 		if(done<=0) return(done);
 		buf+=done;
 		todo-=done;
@@ -286,8 +303,8 @@ void dosigterm(int sig)
 		shutdown(sockinet,2);
 		close(sockinet);
 	}
-	fclose(pidfile);
-	(void) unlink(PIDFILE);
+	fclose(pidf);
+	(void) unlink(pidfile);
 	if(clin>0 && hw.deinit_func) hw.deinit_func();
 #ifdef USE_SYSLOG
 	closelog();
@@ -466,7 +483,7 @@ void add_client(int sock)
 	if(client_addr.sa_family==AF_UNIX)
 	{
 		cli_type[clin]=CT_LOCAL;
-		logprintf(LOG_NOTICE,"accepted new client on %s",LIRCD);
+		logprintf(LOG_NOTICE,"accepted new client on %s",lircdfile);
 	}
 	else if(client_addr.sa_family==AF_INET)
 	{
@@ -673,11 +690,11 @@ void start_server(mode_t permission,int nodaemon)
 	int fd;
 	
 	/* create pid lockfile in /var/run */
-	if((fd=open(PIDFILE,O_RDWR|O_CREAT,0644))==-1 ||
-	   (pidfile=fdopen(fd,"r+"))==NULL)
+	if((fd=open(pidfile,O_RDWR|O_CREAT,0644))==-1 ||
+	   (pidf=fdopen(fd,"r+"))==NULL)
 	{
 		fprintf(stderr,"%s: can't open or create %s\n",
-			progname,PIDFILE);
+			progname,pidfile);
 		perror(progname);
 		exit(EXIT_FAILURE);
 	}
@@ -685,26 +702,26 @@ void start_server(mode_t permission,int nodaemon)
 	{
 		pid_t otherpid;
 		
-		if(fscanf(pidfile,"%d\n",&otherpid)>0)
+		if(fscanf(pidf,"%d\n",&otherpid)>0)
 		{
 			fprintf(stderr,"%s: there seems to already be "
 				"a lircd process with pid %d\n",
 				progname,otherpid);
 			fprintf(stderr,"%s: otherwise delete stale "
-				"lockfile %s\n",progname,PIDFILE);
+				"lockfile %s\n",progname,pidfile);
 		}
 		else
 		{
 			fprintf(stderr,"%s: invalid %s encountered\n",
-				progname,PIDFILE);
+				progname,pidfile);
 		}
 		exit(EXIT_FAILURE);
 	}
 	(void) fcntl(fd,F_SETFD,FD_CLOEXEC);
-	rewind(pidfile);
-	(void) fprintf(pidfile,"%d\n",getpid());
-	(void) fflush(pidfile);
-	(void) ftruncate(fileno(pidfile),ftell(pidfile));
+	rewind(pidf);
+	(void) fprintf(pidf,"%d\n",getpid());
+	(void) fflush(pidf);
+	(void) ftruncate(fileno(pidf),ftell(pidf));
 
 	/* create socket*/
 	sockfd=socket(AF_UNIX,SOCK_STREAM,0);
@@ -712,68 +729,55 @@ void start_server(mode_t permission,int nodaemon)
 	{
 		fprintf(stderr,"%s: could not create socket\n",progname);
 		perror(progname);
-		fclose(pidfile);
-		(void) unlink(PIDFILE);
-		exit(EXIT_FAILURE);
-	};
+		goto start_server_failed0;
+	}
 	
 	/* 
 	   get owner, permissions, etc.
 	   so new socket can be the same since we
 	   have to delete the old socket.  
 	*/
-	ret=stat(LIRCD,&s);
+	ret=stat(lircdfile,&s);
 	if(ret==-1 && errno!=ENOENT)
 	{
 		fprintf(stderr,"%s: could not get file information for %s\n",
-			progname,LIRCD);
+			progname,lircdfile);
 		perror(progname);
-		close(sockfd);
-		fclose(pidfile);
-		(void) unlink(PIDFILE);
-		exit(EXIT_FAILURE);
+		goto start_server_failed1;
 	}
 	if(ret!=-1)
 	{
 		new=0;
-		ret=unlink(LIRCD);
+		ret=unlink(lircdfile);
 		if(ret==-1)
 		{
 			fprintf(stderr,"%s: could not delete %s\n",
-				progname,LIRCD);
+				progname,lircdfile);
 			perror(NULL);
-			close(sockfd);
-			fclose(pidfile);
-			(void) unlink(PIDFILE);
-			exit(EXIT_FAILURE);
+			goto start_server_failed1;
 		}
 	}
 	
 	serv_addr.sun_family=AF_UNIX;
-	strcpy(serv_addr.sun_path,LIRCD);
+	strcpy(serv_addr.sun_path,lircdfile);
 	if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr))==-1)
 	{
 		fprintf(stderr,"%s: could not assign address to socket\n",
 			progname);
 		perror(progname);
-		close(sockfd);
-		fclose(pidfile);
-		(void) unlink(PIDFILE);
-		exit(EXIT_FAILURE);
+		goto start_server_failed1;
 	}
 	
 	if(new ?
-	   chmod(LIRCD,permission):
-	   (chmod(LIRCD,s.st_mode)==-1 || chown(LIRCD,s.st_uid,s.st_gid)==-1)
+	   chmod(lircdfile,permission):
+	   (chmod(lircdfile,s.st_mode)==-1 ||
+	    chown(lircdfile,s.st_uid,s.st_gid)==-1)
 	   )
 	{
 		fprintf(stderr,"%s: could not set file permissions\n",
 			progname);
 		perror(progname);
-		close(sockfd);
-		fclose(pidfile);
-		(void) unlink(PIDFILE);
-		exit(EXIT_FAILURE);
+		goto start_server_failed1;
 	}
 	
 	listen(sockfd,3);
@@ -790,11 +794,8 @@ void start_server(mode_t permission,int nodaemon)
 			fprintf(stderr,"%s: could not create TCP/IP socket\n",
 				progname);
 			perror(progname);
-			close(sockfd);
-			fclose(pidfile);
-			(void) unlink(PIDFILE);
-			exit(EXIT_FAILURE);
-		};
+			goto start_server_failed1;
+		}
 		(void) setsockopt(sockinet,SOL_SOCKET,SO_REUSEADDR,
 				  &enable,sizeof(enable));
 		serv_addr_in.sin_family=AF_INET;
@@ -804,14 +805,11 @@ void start_server(mode_t permission,int nodaemon)
 		if(bind(sockinet,(struct sockaddr *) &serv_addr_in,
 			sizeof(serv_addr_in))==-1)
 		{
-			fprintf(stderr,"%s: could not assign address to socket\n",
+			fprintf(stderr,
+				"%s: could not assign address to socket\n",
 				progname);
 			perror(progname);
-			close(sockinet);
-			close(sockfd);
-			fclose(pidfile);
-			(void) unlink(PIDFILE);
-			exit(EXIT_FAILURE);
+			goto start_server_failed2;
 		}
 		
 		listen(sockinet,3);
@@ -822,14 +820,14 @@ void start_server(mode_t permission,int nodaemon)
 #ifdef DAEMONIZE
 	if(nodaemon)
 	{
-		openlog(progname,LOG_CONS|LOG_PID|LOG_PERROR,LIRC_SYSLOG);
+		openlog(syslogident,LOG_CONS|LOG_PID|LOG_PERROR,LIRC_SYSLOG);
 	}
 	else
 	{
-		openlog(progname,LOG_CONS|LOG_PID,LIRC_SYSLOG);
+		openlog(syslogident,LOG_CONS|LOG_PID,LIRC_SYSLOG);
 	}
 #else
-	openlog(progname,LOG_CONS|LOG_PID|LOG_PERROR,LIRC_SYSLOG);
+	openlog(syslogident,LOG_CONS|LOG_PID|LOG_PERROR,LIRC_SYSLOG);
 #endif
 #else
 	lf=fopen(logfile,"a");
@@ -837,18 +835,24 @@ void start_server(mode_t permission,int nodaemon)
 	{
 		fprintf(stderr,"%s: could not open logfile\n",progname);
 		perror(progname);
-		if(listen_tcpip)
-		{
-			close(sockinet);
-		}
-		close(sockfd);
-		fclose(pidfile);
-		(void) unlink(PIDFILE);
-		exit(EXIT_FAILURE);
+		goto start_server_failed2;
 	}
 	gethostname(hostname,HOSTNAME_LEN);
 #endif
 	LOGPRINTF(1,"started server socket");
+	return;
+
+ start_server_failed2:
+	if(listen_tcpip)
+	{
+		close(sockinet);
+	}
+ start_server_failed1:
+	close(sockfd);
+ start_server_failed0:
+	fclose(pidf);
+	(void) unlink(pidfile);
+	exit(EXIT_FAILURE);
 }
 
 #ifndef USE_SYSLOG
@@ -903,10 +907,10 @@ void daemonize(void)
 		dosigterm(SIGTERM);
 	}
 	umask(0);
-	rewind(pidfile);
-	(void) fprintf(pidfile,"%d\n",getpid());
-	(void) fflush(pidfile);
-	(void) ftruncate(fileno(pidfile),ftell(pidfile));
+	rewind(pidf);
+	(void) fprintf(pidf,"%d\n",getpid());
+	(void) fflush(pidf);
+	(void) ftruncate(fileno(pidf),ftell(pidf));
 	daemonized=1;
 }
 
@@ -1105,7 +1109,7 @@ int send_remote_list(int fd,char *message)
 	while(all)
 	{
 		len=snprintf(buffer,PACKET_SIZE+1,"%s\n",all->name);
-		if(len==PACKET_SIZE+1)
+		if(len>=PACKET_SIZE+1)
 		{
 			len=sprintf(buffer,"name_too_long\n");
 		}
@@ -1158,7 +1162,7 @@ int send_remote(int fd,char *message,struct ir_remote *remote)
 			     codes->code,
 			     codes->name);
 #endif
-		if(len==PACKET_SIZE+1)
+		if(len>=PACKET_SIZE+1)
 		{
 			len=sprintf(buffer,"code_too_long\n");
 		}
@@ -1189,7 +1193,7 @@ int send_name(int fd,char *message,struct ir_ncode *code)
 		     code->code,
 		     code->name);
 #endif
-	if(len==PACKET_SIZE+1)
+	if(len>=PACKET_SIZE+1)
 	{
 		len=sprintf(buffer,"1\ncode_too_long\n");
 	}
@@ -1215,6 +1219,62 @@ int list(int fd,char *message,char *arguments)
 	return(send_name(fd,message,code));
 }
 
+int set_transmitters(int fd,char *message,char *arguments)
+{
+	char *next_arg=NULL,*end_ptr;
+	unsigned long next_tx_int;
+	unsigned long next_tx_hex;
+	unsigned int channels=0;
+	int retval=0;
+	int i;
+	
+	if(arguments==NULL) goto string_error;
+	if(hw.send_mode==0) return(send_error(fd,message,"hardware does not "
+					      "support sending\n"));
+	if(hw.ioctl_func == NULL ||
+	   !(hw.features & LIRC_CAN_SET_TRANSMITTER_MASK))
+	{
+		return(send_error(fd,message,"hardware does not support "
+				  "multiple transmitters\n"));
+	}
+	
+	next_arg=strtok(arguments,WHITE_SPACE);
+	if (next_arg==NULL) goto string_error;
+	do
+	{
+		next_tx_int=-1;
+		next_tx_int = strtoul(next_arg,&end_ptr,10);
+		if(*end_ptr || (next_tx_int == ULONG_MAX && errno == ERANGE))
+		{
+			return(send_error(fd,message, "invalid argument\n"));
+		}
+		if(next_tx_int > MAX_TX)
+		{
+			return(send_error(fd, message, "cannot support more "
+					  "than %d transmitters\n", MAX_TX));
+		}
+		next_tx_hex=1;
+		for(i=1; i<next_tx_int; i++) next_tx_hex = next_tx_hex << 1;
+		channels |= next_tx_hex;
+	}while ((next_arg=strtok(NULL,WHITE_SPACE))!=NULL);
+	
+	retval = hw.ioctl_func(LIRC_SET_TRANSMITTER_MASK, channels);
+	if(retval<0)
+	{
+		return(send_error(fd, message, "error - could not set "
+				  "transmitters\n"));
+	}
+	if (retval>0)
+	{
+		return(send_error(fd, message, "error - maximum of %d "
+				  "transmitters\n", retval));
+	}
+	return(send_success(fd,message));
+	
+string_error:
+	return(send_error(fd,message,"no arguments given\n"));
+}
+
 int send_once(int fd,char *message,char *arguments)
 {
 	return(send_core(fd,message,arguments,1));
@@ -1266,7 +1326,7 @@ int send_core(int fd,char *message,char *arguments,int once)
 	}
 	if(once)
 	{
-		remote->repeat_countdown=max(remote->min_repeat,reps);
+		remote->repeat_countdown=max(remote->repeat_countdown,reps);
 	}
 	else
 	{
@@ -1636,8 +1696,8 @@ int waitfordata(long maxusec)
 			gettimeofday(&start,NULL);
 			if(maxusec>0)
 			{
-				tv.tv_sec=0;
-				tv.tv_usec=maxusec;
+				tv.tv_sec= maxusec / 1000000;
+				tv.tv_usec=maxusec % 1000000;
 			}
 #ifdef SIM_REC
 			ret=select(maxfd+1,&fds,NULL,NULL,NULL);
@@ -1767,6 +1827,7 @@ int main(int argc,char **argv)
 	struct sigaction act;
 	int nodaemon=0;
 	mode_t permission=S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
+	char *device=NULL;
 
 	hw_choose_driver(NULL);
 	while(1)
@@ -1782,16 +1843,24 @@ int main(int argc,char **argv)
 			{"device",required_argument,NULL,'d'},
 			{"listen",optional_argument,NULL,'l'},
 			{"connect",required_argument,NULL,'c'},
+                        {"output",required_argument,NULL,'o'},
+                        {"pidfile",required_argument,NULL,'P'},
+#                       ifndef USE_SYSLOG
+                        {"logfile",required_argument,NULL,'L'},
+#                       endif
 #                       ifdef DEBUG
 			{"debug",optional_argument,NULL,'D'},
 #                       endif
 			{0, 0, 0, 0}
 		};
-#               ifdef DEBUG
-		c = getopt_long(argc,argv,"hvnp:H:d:l::c:D::",long_options,NULL);
-#               else
-		c = getopt_long(argc,argv,"hvnp:H:d:l::c:",long_options,NULL);
-#               endif
+		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:"
+#                               ifndef USE_SYSLOG
+				"L:"
+#                               endif
+#                               ifdef DEBUG
+				"D::"
+#                               endif
+				,long_options,NULL);
 		if(c==-1)
 			break;
 		switch (c)
@@ -1805,13 +1874,18 @@ int main(int argc,char **argv)
 			printf("\t -H --driver=driver\t\tuse given driver\n");
 			printf("\t -d --device=device\t\tread from given device\n");
 			printf("\t -l --listen[=port]\t\tlisten for network connections on port\n");
-			printf("\t -c --connect=host[:port]\t\tconnect to remote lircd server\n");
+			printf("\t -c --connect=host[:port]\tconnect to remote lircd server\n");
+                        printf("\t -o --output=socket\t\toutput socket filename\n");
+                        printf("\t -P --pidfile=file\t\tdaemon pid file\n");
+#                       ifndef USE_SYSLOG
+                        printf("\t -L --logfile=file\t\tdaemon log file\n");
+#                       endif
 #                       ifdef DEBUG
 			printf("\t -D[debug_level] --debug[=debug_level]\n");
 #                       endif
 			return(EXIT_SUCCESS);
 		case 'v':
-			printf("%s\n",progname);
+			printf("%s %s\n",progname,VERSION);
 			return(EXIT_SUCCESS);
 		case 'n':
 			nodaemon=1;
@@ -1833,8 +1907,19 @@ int main(int argc,char **argv)
 			}
 			break;
 		case 'd':
-			hw.device=optarg;
+			device=optarg;
 			break;
+                case 'P':
+                        pidfile=optarg;
+                        break;
+#                       ifndef USE_SYSLOG
+                case 'L':
+                        logfile=optarg;
+                        break;
+#                       endif
+                case 'o':
+                        lircdfile=optarg;
+                        break;
 		case 'l':
 			listen_tcpip=1;
 			if(optarg)
@@ -1886,12 +1971,24 @@ int main(int argc,char **argv)
 		return(EXIT_FAILURE);
 	}
 	
+	if(device!=NULL)
+	{
+		hw.device=device;
+	}
 	if(strcmp(hw.name, "null")==0 && peern==0)
 	{
 		fprintf(stderr,"%s: there's no hardware I can use and "
 			"no peers are specified\n",progname);
 		return(EXIT_FAILURE);
 	}
+	if(hw.device!=NULL && strcmp(hw.device, lircdfile)==0)
+	{
+		fprintf(stderr, "%s: refusing to connect to myself\n",
+			progname);
+		fprintf(stderr, "%s: device and output must not be the "
+			"same file: %s\n", progname, lircdfile);
+		return(EXIT_FAILURE);
+	}
 
 	signal(SIGPIPE,SIG_IGN);
 	
