@@ -1,4 +1,4 @@
-/*      $Id: lircmd.c,v 5.15 2004/04/25 16:29:24 lirc Exp $      */
+/*      $Id: lircmd.c,v 5.18 2005/10/30 11:11:46 lirc Exp $      */
 
 /****************************************************************************
  ** lircmd.c ****************************************************************
@@ -68,7 +68,7 @@ enum directive {move_n,move_ne,move_e,move_se,
 		button1_down,button1_up,button1_toggle,button1_click,
 		button2_down,button2_up,button2_toggle,button2_click,
 		button3_down,button3_up,button3_toggle,button3_click,
-		mouse_activate,mouse_toggle_activate
+		mouse_activate,mouse_toggle_activate,ignore
 };
 
 struct config_mouse
@@ -102,7 +102,8 @@ struct config_mouse config_table[]=
 	{"BUTTON3_UP"         ,button3_up      , 0, 0, 0,      0,BUTTON3, 0},
 	{"BUTTON3_TOGGLE"     ,button3_toggle  , 0, 0, 0,BUTTON3,BUTTON3, 1},
 	{"BUTTON3_CLICK"      ,button3_click   , 0, 0, 0,BUTTON3,BUTTON3, 0},
-	{NULL                 ,button3_click   , 0, 0, 0,      0,      0, 0}
+	{"IGNORE"             ,ignore          , 0, 0, 0,      0,      0, 0},
+	{NULL                 ,ignore          , 0, 0, 0,      0,      0, 0}
 };
 
 enum protocol {mouse_systems,imps_2,im_serial};
@@ -134,7 +135,8 @@ struct state_mouse new_ms,ms=
 	{button_up,button_up,button_up}
 };
 
-char *progname="lircmd "VERSION;
+char *progname="lircmd";
+static const char *syslogident = "lircmd-" VERSION;
 char *configfile=LIRCMDCFGFILE;
 
 int lircd,lircm;
@@ -414,7 +416,8 @@ void mouse_conv(int rep,char *button,char *remote)
 			}
 		}
 		
-		if(ms.active || ms.always_active)
+		if(tm->tm_directive!=ignore &&
+		   (ms.active || ms.always_active))
 		{
 			int i;
 			for(i=0;config_table[i].string!=NULL;i++)
@@ -736,7 +739,7 @@ int main(int argc,char **argv)
 			printf("\t -n --nodaemon\t\tdon't fork to background\n");
 			return(EXIT_SUCCESS);
 		case 'v':
-			printf("%s\n",progname);
+			printf("%s %s\n",progname,VERSION);
 			return(EXIT_SUCCESS);
 		case 'n':
 			nodaemon=1;
@@ -817,7 +820,7 @@ int main(int argc,char **argv)
 #ifdef DAEMONIZE
 	if(!nodaemon) daemonize();
 #endif
-	openlog(progname,LOG_CONS,LOG_DAEMON);
+	openlog(syslogident,LOG_CONS,LOG_DAEMON);
 	
 	signal(SIGPIPE,SIG_IGN);
 
