@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.21 2004/08/28 10:07:27 lirc Exp $      */
+/*      $Id: receive.c,v 5.29 2005/10/20 18:43:36 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -32,11 +32,13 @@ inline lirc_t lirc_t_max(lirc_t a,lirc_t b)
 
 inline void set_pending_pulse(lirc_t deltap)
 {
+	LOGPRINTF(5, "pending pulse: %lu", deltap);
 	rec_buffer.pendingp=deltap;
 }
 
 inline void set_pending_space(lirc_t deltas)
 {
+	LOGPRINTF(5, "pending space: %lu", deltas);
 	rec_buffer.pendings=deltas;
 }
 
@@ -44,7 +46,9 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 {
 	if(rec_buffer.rptr<rec_buffer.wptr)
 	{
-		LOGPRINTF(3,"<%lu",(unsigned long)
+		LOGPRINTF(3,"<%c%lu",
+			  rec_buffer.data[rec_buffer.rptr]&PULSE_BIT ?
+			  'p':'s', (unsigned long)
 			  rec_buffer.data[rec_buffer.rptr]&(PULSE_MASK));
 		rec_buffer.sum+=rec_buffer.data[rec_buffer.rptr]&(PULSE_MASK);
 		return(rec_buffer.data[rec_buffer.rptr++]);
@@ -58,6 +62,7 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 			data=hw.readdata(2*maxusec<50000 ? 50000:2*maxusec);
 			if(!data)
 			{
+				LOGPRINTF(3,"timeout: %u", maxusec);
 				return 0;
 			}
 
@@ -67,7 +72,9 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 				&(PULSE_MASK);
                         rec_buffer.wptr++;
                         rec_buffer.rptr++;
-			LOGPRINTF(3,"+%lu",(unsigned long)
+			LOGPRINTF(3,"+%c%lu",
+				  rec_buffer.data[rec_buffer.rptr-1]&
+				  PULSE_BIT ? 'p':'s', (unsigned long)
 				  rec_buffer.data[rec_buffer.rptr-1]
 				  &(PULSE_MASK));
                         return(rec_buffer.data[rec_buffer.rptr-1]);
@@ -146,14 +153,13 @@ int clear_rec_buffer(void)
 		else
 		{
 			rec_buffer.wptr=0;
+			data=hw.readdata(0);
+			
+			LOGPRINTF(3,"c%lu",(unsigned long) data&(PULSE_MASK));
+			
+			rec_buffer.data[rec_buffer.wptr]=data;
+			rec_buffer.wptr++;
 		}
-		
-		data=hw.readdata(0);
-		
-		LOGPRINTF(3,"c%lu",(unsigned long) data&(PULSE_MASK));
-		
-		rec_buffer.data[rec_buffer.wptr]=data;
-		rec_buffer.wptr++;
 	}
 
 	rewind_rec_buffer();
@@ -164,6 +170,7 @@ int clear_rec_buffer(void)
 
 inline void unget_rec_buffer(int count)
 {
+	LOGPRINTF(5, "unget: %d", count);
 	if(count==1 || count==2)
 	{
 		rec_buffer.rptr-=count;
@@ -176,6 +183,13 @@ inline void unget_rec_buffer(int count)
 	}
 }
 
+inline void unget_rec_buffer_delta(lirc_t delta)
+{
+	rec_buffer.rptr--;
+	rec_buffer.sum-=delta&(PULSE_MASK);
+	rec_buffer.data[rec_buffer.rptr]=delta;
+}
+
 inline lirc_t get_next_pulse(lirc_t maxusec)
 {
 	lirc_t data;
@@ -237,14 +251,15 @@ int expectpulse(struct ir_remote *remote,int exdelta)
 	lirc_t deltap;
 	int retval;
 	
+	LOGPRINTF(5, "expecting pulse: %lu", exdelta);
 	if(!sync_pending_space(remote)) return 0;
 	
 	deltap=get_next_pulse(rec_buffer.pendingp+exdelta);
 	if(deltap==0) return(0);
 	if(rec_buffer.pendingp>0)
 	{
-		retval=expect(remote,deltap,
-			      rec_buffer.pendingp+exdelta);
+		if(rec_buffer.pendingp>deltap) return 0;
+		retval=expect(remote,deltap-rec_buffer.pendingp,exdelta);
 		if(!retval) return(0);
 		set_pending_pulse(0);
 	}
@@ -260,14 +275,15 @@ int expectspace(struct ir_remote *remote,int exdelta)
 	lirc_t deltas;
 	int retval;
 
+	LOGPRINTF(5, "expecting space: %lu", exdelta);
 	if(!sync_pending_pulse(remote)) return 0;
 	
 	deltas=get_next_space(rec_buffer.pendings+exdelta);
 	if(deltas==0) return(0);
 	if(rec_buffer.pendings>0)
 	{
-		retval=expect(remote,deltas,
-			      rec_buffer.pendings+exdelta);
+		if(rec_buffer.pendings>deltas) return 0;
+		retval=expect(remote,deltas-rec_buffer.pendings,exdelta);
 		if(!retval) return(0);
 		set_pending_space(0);
 	}
@@ -308,6 +324,19 @@ inline int expectone(struct ir_remote *remote,int bit)
 			set_pending_pulse(remote->pone);
 		}
 	}
+	else if(is_space_first(remote))
+	{
+		if(remote->sone>0 && !expectspace(remote,remote->sone))
+		{
+			unget_rec_buffer(1);
+			return(0);
+		}
+		if(remote->pone>0 && !expectpulse(remote,remote->pone))
+		{
+			unget_rec_buffer(2);
+			return(0);
+		}
+	}
 	else
 	{
 		if(remote->pone>0 && !expectpulse(remote,remote->pone))
@@ -361,6 +390,19 @@ inline int expectzero(struct ir_remote *remote,int bit)
 			set_pending_space(remote->szero);
 		}
 	}
+	else if(is_space_first(remote))
+	{
+		if(remote->szero>0 && !expectspace(remote,remote->szero))
+		{
+			unget_rec_buffer(1);
+			return(0);
+		}
+		if(remote->pzero>0 && !expectpulse(remote,remote->pzero))
+		{
+			unget_rec_buffer(2);
+			return(0);
+		}
+	}
 	else
 	{
 		if(!expectpulse(remote,remote->pzero))
@@ -395,9 +437,8 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 	
 	if(last_remote!=NULL && !is_rcmm(remote))
 	{
-		while(deltas<last_remote->remaining_gap*
-		      (100-last_remote->eps)/100 &&
-		      deltas<last_remote->remaining_gap-last_remote->aeps)
+		while(!expect_at_least(last_remote, deltas,
+				       last_remote->remaining_gap))
 		{
 			deltap=get_next_pulse(1000000);
 			if(deltap==0) return(0);
@@ -412,9 +453,8 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 		}
 		if(has_toggle_mask(remote))
 		{
-			if(deltas>last_remote->remaining_gap*
-			   (100+last_remote->eps)/100 &&
-			   deltas>last_remote->remaining_gap+last_remote->aeps)
+			if(!expect_at_most(last_remote, deltas,
+					   last_remote->remaining_gap))
 			{
 				remote->toggle_mask_state=0;
 				remote->toggle_code=NULL;
@@ -508,7 +548,7 @@ inline int get_trail(struct ir_remote *remote)
 	}
 	if(rec_buffer.pendingp>0)
 	{
-		if(!expectpulse(remote,0)) return(0);
+		if(!sync_pending_pulse(remote)) return(0);
 	}
 	return(1);
 }
@@ -517,7 +557,7 @@ inline int get_gap(struct ir_remote *remote,lirc_t gap)
 {
 	lirc_t data;
 	
-	LOGPRINTF(2,"sum: %ld",rec_buffer.sum);
+	LOGPRINTF(2,"sum: %d",rec_buffer.sum);
 	data=get_next_rec_buffer(gap*(100-remote->eps)/100);
 	if(data==0) return(1);
 	if(!is_space(data))
@@ -526,8 +566,7 @@ inline int get_gap(struct ir_remote *remote,lirc_t gap)
 		return(0);
 	}
 	unget_rec_buffer(1);
-	if(data<gap*(100-remote->eps)/100 &&
-	   data<gap-remote->aeps)
+	if(!expect_at_least(remote, data, gap))
 	{
 		LOGPRINTF(1,"end of signal not found");
 		return(0);
@@ -610,7 +649,7 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			}
 			else
 			{
-				LOGPRINTF(2,"no match for %ld+%ld=%ld",
+				LOGPRINTF(2,"no match for %d+%d=%d",
 					 deltap,deltas,sum);
 				return((ir_code) -1);
 			}
@@ -664,7 +703,7 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			}
 			else
 			{
-				LOGPRINTF(2,"no match for %ld+%ld=%ld",
+				LOGPRINTF(2,"no match for %d+%d=%d",
 					 deltas,deltap,sum);
 				return((ir_code) -1);
 			}
@@ -705,9 +744,11 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 	else if(is_serial(remote))
 	{
 		int received;
-		int space, start_bit, stop_bit;
-		lirc_t delta,origdelta,pending,expecting;
+		int space, start_bit, stop_bit, parity_bit;
+		int parity;
+		lirc_t delta,origdelta,pending,expecting, gap_delta;
 		lirc_t base, stop;
+		lirc_t max_space, max_pulse;
 		
 		base=1000000/remote->baud;
 		
@@ -718,26 +759,34 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		space=(rec_buffer.pendingp==0); /* expecting space ? */
 		start_bit=0;
 		stop_bit=0;
+		parity_bit=0;
 		delta=origdelta=0;
 		stop=base*remote->stop_bits/2;
+		parity=0;
+		gap_delta=0;
 		
-		while(received<bits)
+		max_space = remote->sone*remote->bits_in_byte+stop;
+		max_pulse = remote->pzero*(1+remote->bits_in_byte);
+		if(remote->parity != IR_PARITY_NONE)
+		{
+			parity_bit = 1;
+			max_space += remote->sone;
+			max_pulse += remote->pzero;
+			bits += bits/remote->bits_in_byte;
+		}
+		
+		while(received<bits || stop_bit)
 		{
 			if(delta==0)
 			{
 				delta=space ?
-					get_next_space(remote->sone*
-						       remote->bits_in_byte+
-						       base*remote->stop_bits/2):
-					get_next_pulse(remote->pzero*
-						       remote->bits_in_byte);
+					get_next_space(max_space):
+					get_next_pulse(max_pulse);
 				if(delta==0 && space &&
-				   received+remote->bits_in_byte>=bits)
+				   received+remote->bits_in_byte+parity_bit>=bits)
 				{
 					/* open end */
-					delta=remote->sone*
-						remote->bits_in_byte+
-						base*remote->stop_bits/2;
+					delta=max_space;
 				}
 				origdelta=delta;
 			}
@@ -749,26 +798,29 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			}
 			pending=(space ?
 				 rec_buffer.pendings:rec_buffer.pendingp);
-			if(delta<=pending)
+			if(expect(remote, delta, pending))
 			{
-				if(!expect(remote, delta, pending))
-				{
-					LOGPRINTF(1,"failed before bit %d",
-						  received+1);
-					return((ir_code) -1);
-				}
 				delta=0;
 			}
+			else if(delta>pending)
+			{
+				delta-=pending;
+			}
 			else
 			{
-				delta-=pending;
+				LOGPRINTF(1,"failed before bit %d",
+					  received+1);
+				return((ir_code) -1);
 			}
 			if(pending>0)
 			{
 				if(stop_bit)
 				{
+					LOGPRINTF(5, "delta: %lu", delta);
+					gap_delta = delta;
 					delta=0;
 					set_pending_pulse(base);
+					set_pending_space(0);
 					stop_bit=0;
 					space=0;
 					LOGPRINTF(3,"stop bit found");
@@ -792,15 +844,28 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 				received++;
 				code<<=1;
 				code|=space;
+				parity^=space;
 				LOGPRINTF(2,"adding %d",space);
-				if(received%remote->bits_in_byte==0)
+				if(received%(remote->bits_in_byte+parity_bit)==0)
 				{
 					ir_code temp;
 					
-					temp=code>>remote->bits_in_byte;
+					if((remote->parity == IR_PARITY_EVEN && parity) ||
+					   (remote->parity == IR_PARITY_ODD && !parity))
+					{
+						LOGPRINTF(1, "parity error "
+							  "after %d bits",
+							  received+1);
+						return((ir_code) -1);
+					}
+					parity = 0;
+					
+					/* parity bit is filtered out */
+					temp=code>>(remote->bits_in_byte+parity_bit);
 					code=temp<<remote->bits_in_byte|
-						reverse(code,remote->bits_in_byte);
-
+						reverse(code>>parity_bit,
+							remote->bits_in_byte);
+					
 					if(space && delta==0)
 					{
 						LOGPRINTF(1,"failed at stop "
@@ -828,6 +893,8 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 				space=(space ? 0:1);
 			}
 		}
+		if(gap_delta) unget_rec_buffer_delta(gap_delta);
+		set_pending_pulse(0);
 		set_pending_space(0);
 		return code;
 	}
@@ -992,8 +1059,7 @@ int receive_decode(struct ir_remote *remote,
 			{
 				header=0;
 				if(!(remote->flags&NO_HEAD_REP && 
-				     (sync<=remote->gap+remote->gap*remote->eps/100
-				      || sync<=remote->gap+remote->aeps)))
+				     expect_at_most(remote,sync,remote->gap)))
 				{
 					LOGPRINTF(1,"failed on header");
 					return(0);
@@ -1192,8 +1258,8 @@ int receive_decode(struct ir_remote *remote,
 		} /* end of mode specific code */
 	}
 	*prep=pre;*codep=code;*postp=post;
-	if(sync<=remote->remaining_gap*(100+remote->eps)/100
-	   || sync<=remote->remaining_gap+remote->aeps)
+	if(!has_repeat(remote) &&
+	   expect_at_most(remote, sync, remote->remaining_gap))
 		*repeat_flagp=1;
 	else
 		*repeat_flagp=0;
