@@ -1,4 +1,4 @@
-/*      $Id: serial.c,v 5.8 2001/10/18 16:15:36 lirc Exp $      */
+/*      $Id: serial.c,v 5.11 2005/03/07 09:33:48 lirc Exp $      */
 
 /****************************************************************************
  ** serial.c ****************************************************************
@@ -133,6 +133,46 @@ int tty_setbaud(int fd,int baud)
 	return(1);
 }
 
+int tty_setcsize(int fd,int csize)
+{
+	struct termios options;
+	int size;
+
+	switch(csize)
+	{
+	case 5:
+		size=CS5;
+		break;
+	case 6:
+		size=CS6;
+		break;
+	case 7:
+                size=CS7;
+                break;
+	case 8:
+                size=CS8;
+                break;
+	default:
+		LOGPRINTF(1,"tty_setcsize(): bad csize rate %d",csize);
+		return(0);
+	}		
+	if(tcgetattr(fd, &options)==-1)
+	{
+		LOGPRINTF(1,"tty_setcsize(): tcgetattr() failed");
+		LOGPERROR(1,"tty_setcsize()");
+		return(0);
+	}
+	options.c_cflag &= ~CSIZE;
+	options.c_cflag |= size;
+	if(tcsetattr(fd,TCSAFLUSH,&options)==-1)
+	{
+		LOGPRINTF(1,"tty_setcsize(): tcsetattr() failed");
+		LOGPERROR(1,"tty_setcsize()");
+		return(0);
+	}
+	return(1);
+}
+
 int tty_create_lock(char *name)
 {
 	char filename[FILENAME_MAX+1];
@@ -153,7 +193,7 @@ int tty_create_lock(char *name)
 	
 	if(strlen(filename)+strlen(s)>FILENAME_MAX)
 	{
-		logprintf(LOG_ERR,"%s: invalid filename \"%s%s\"",
+		logprintf(LOG_ERR,"invalid filename \"%s%s\"",
 			  filename,s);
 		return(0);
 	}
@@ -223,7 +263,7 @@ int tty_create_lock(char *name)
 	}
 	if(write(lock,id,len)!=len)
 	{
-		logprintf(LOG_ERR,"%s: could not write pid to lock file");
+		logprintf(LOG_ERR,"could not write pid to lock file");
 		logperror(LOG_ERR,NULL);
 		close(lock);
 		if(unlink(filename)==-1)
@@ -279,7 +319,7 @@ int tty_create_lock(char *name)
 				logperror(LOG_ERR,NULL);
 				if(unlink(filename)==-1)
 				{
-					logprintf(LOG_ERR,"%s: could not delete "
+					logprintf(LOG_ERR,"could not delete "
 						  "file \"%s\"",filename);
 					logperror(LOG_ERR,NULL);
 				        /* FALLTHROUGH */
@@ -319,8 +359,8 @@ int tty_create_lock(char *name)
 		{
 			if(chdir(cwd)==-1)
 			{
-				logprintf(LOG_ERR,"chdir() to \"%s\" "
-					  "failded ",cwd);
+				logprintf(LOG_ERR,"chdir() to \"%s\" failed",
+					  cwd);
 				logperror(LOG_ERR,NULL);
 				if(unlink(filename)==-1)
 				{
@@ -434,7 +474,7 @@ int tty_write(int fd,char byte)
 	   (for 9600 baud rate, it takes about 100 msec */
 	usleep(100*1000);
 	
-	/* we dont wait because tcdrain() does this for us */
+	/* we don't wait because tcdrain() does this for us */
 	/* tcdrain(fd); */ 
 	/* but unfortunately this does not seem to be
 	   implemented in 2.0.x kernels ... */
