@@ -1,4 +1,4 @@
-/*      $Id: lirc_parallel.c,v 5.27 2004/11/07 13:31:18 lirc Exp $      */
+/*      $Id: lirc_parallel.c,v 5.33 2005/09/22 20:10:37 lirc Exp $      */
 
 /****************************************************************************
  ** lirc_parallel.c *********************************************************
@@ -77,6 +77,16 @@
 
 #define LIRC_DRIVER_NAME "lirc_parallel"
 
+#ifndef LIRC_IRQ
+#define LIRC_IRQ 7
+#endif
+#ifndef LIRC_PORT
+#define LIRC_PORT 0x378
+#endif
+#ifndef LIRC_TIMER
+#define LIRC_TIMER 65536
+#endif
+
 /***********************************************************************
  *************************   Globale Variablen   ***********************
  ***********************************************************************/
@@ -384,23 +394,24 @@ static loff_t lirc_lseek(struct file *filep,loff_t offset,int orig)
 
 static ssize_t lirc_read(struct file *filep,char *buf,size_t n,loff_t *ppos)
 {
-	int result;
+	int result=0;
 	int count=0;
 	DECLARE_WAITQUEUE(wait, current);
 	
 	if(n%sizeof(lirc_t)) return(-EINVAL);
 	
-	result=verify_area(VERIFY_WRITE,buf,n);
-	if(result) return(result);
-	
 	add_wait_queue(&lirc_wait,&wait);
-	current->state=TASK_INTERRUPTIBLE;
+	set_current_state(TASK_INTERRUPTIBLE);
 	while(count<n)
 	{
 		if(rptr!=wptr)
 		{
-			copy_to_user(buf+count,(char *) &rbuf[rptr],
-				     sizeof(lirc_t));
+			if(copy_to_user(buf+count,(char *) &rbuf[rptr],
+					sizeof(lirc_t)))
+			{
+				result = -EFAULT;
+				break;
+			}
 			rptr=(rptr+1)&(RBUF_SIZE-1);
 			count+=sizeof(lirc_t);
 		}
@@ -417,18 +428,18 @@ static ssize_t lirc_read(struct file *filep,char *buf,size_t n,loff_t *ppos)
 				break;
 			}
 			schedule();
-			current->state=TASK_INTERRUPTIBLE;
+			set_current_state(TASK_INTERRUPTIBLE);
 		}
 	}
 	remove_wait_queue(&lirc_wait,&wait);
-	current->state=TASK_RUNNING;
+	set_current_state(TASK_RUNNING);
 	return(count ? count:result);
 }
 
 static ssize_t lirc_write(struct file *filep,const char *buf,size_t n,
 			  loff_t *ppos)
 {
-	int result,count;
+	int count;
 	unsigned int i;
 	unsigned int level,newlevel;
 	unsigned long flags;
@@ -439,14 +450,12 @@ static ssize_t lirc_write(struct file *filep,const char *buf,size_t n,
 		return(-EBUSY);
 	}
 	if(n%sizeof(lirc_t)) return(-EINVAL);
-	result=verify_area(VERIFY_READ,buf,n);
-	if(result) return(result);
 	
 	count=n/sizeof(lirc_t);
 	
 	if(count>WBUF_SIZE || count%2==0) return(-EINVAL);
 	
-	copy_from_user(wbuf,buf,n);
+	if(copy_from_user(wbuf,buf,n)) return -EFAULT;
 	
 #ifdef LIRC_TIMER
 	if(timer==0) /* try again if device is ready */
@@ -570,7 +579,7 @@ static int lirc_open(struct inode* node,struct file* filep)
 	{
 		return(-EBUSY);
 	}
-	pport->ops->enable_irq(pport);
+	parport_enable_irq(pport);
 
 	/* init read ptr */
 	rptr=wptr=0;
@@ -630,6 +639,7 @@ static struct lirc_plugin plugin = {
        set_use_inc:    set_use_inc,
        set_use_dec:    set_use_dec,
        fops:           &lirc_fops,
+       owner:          THIS_MODULE,
 };
 
 #ifdef MODULE
@@ -662,7 +672,7 @@ static void poll_state(unsigned long ignored)
 
 static int pf(void *handle)
 {
-	pport->ops->disable_irq(pport);
+	parport_disable_irq(pport);
 	is_claimed=0;
 	return(0);
 }
@@ -673,7 +683,8 @@ static void kf(void *handle)
 		return;
 	if(!lirc_claim())
 		return;
-	pport->ops->enable_irq(pport);
+	parport_enable_irq(pport);
+	lirc_off();
 	/* this is a bit annoying when you actually print...*/
 	/*
 	printk(KERN_INFO "%s: reclaimed port\n",LIRC_DRIVER_NAME);
@@ -760,7 +771,6 @@ int init_module(void)
   
 void cleanup_module(void)
 {
-	if(MOD_IN_USE) return;
 	parport_unregister_device(ppdevice);
 	lirc_unregister_plugin(plugin.minor);
 }
