@@ -108,13 +108,15 @@
 
 #define TEKRAM_PW 0x10 /* Pulse select bit */
 
-/* 10bit * 1s/115200bit in milli seconds = 87ms*/
+/* 10bit * 1s/115200bit in miliseconds = 87ms*/
 #define TIME_CONST (10000000ul/115200ul)
 
 #endif
 
 #ifdef LIRC_SIR_ACTISYS_ACT200L
 static void init_act200(void);
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+static void init_act220(void);
 #endif
 
 /******************************* SA1100 ********************************/
@@ -152,9 +154,9 @@ static unsigned int duty_cycle = 50;   /* duty cycle of 50% */
 
 #define LIRC_DRIVER_NAME "lirc_sir"
 
-#ifndef LIRC_SIR_TEKRAM
 #define PULSE '['
 
+#ifndef LIRC_SIR_TEKRAM
 /* 9bit * 1s/115200bit in milli seconds = 78.125ms*/
 #define TIME_CONST (9000000ul/115200ul)
 #endif
@@ -165,6 +167,13 @@ static unsigned int duty_cycle = 50;   /* duty cycle of 50% */
 #define SIR_TIMEOUT	(HZ*5/100)
 
 #ifndef LIRC_ON_SA1100
+#ifndef LIRC_IRQ
+#define LIRC_IRQ 4
+#endif
+#ifndef LIRC_PORT
+#define LIRC_PORT 0x3e8
+#endif
+
 static int io = LIRC_PORT;
 static int irq = LIRC_IRQ;
 static int threshold = 3;
@@ -183,10 +192,9 @@ static DECLARE_WAIT_QUEUE_HEAD(lirc_read_queue);
 static spinlock_t hardware_lock = SPIN_LOCK_UNLOCKED;
 static spinlock_t dev_lock = SPIN_LOCK_UNLOCKED;
 
-static lirc_t rx_buf[RBUF_LEN]; unsigned int rx_tail = 0, rx_head = 0;
-#ifndef LIRC_SIR_TEKRAM
+static lirc_t rx_buf[RBUF_LEN]; 
+static unsigned int rx_tail = 0, rx_head = 0;
 static lirc_t tx_buf[WBUF_LEN];
-#endif
 
 static int debug = 0;
 #define dprintk(fmt, args...)                                     \
@@ -214,10 +222,8 @@ static void drop_chrdev(void);
 	/* Hardware */
 static irqreturn_t sir_interrupt(int irq, void * dev_id,
 				 struct pt_regs * regs);
-#ifndef LIRC_SIR_TEKRAM
 static void send_space(unsigned long len);
 static void send_pulse(unsigned long len);
-#endif
 static int init_hardware(void);
 static void drop_hardware(void);
 	/* Initialisation */
@@ -302,19 +308,18 @@ static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 	if(n%sizeof(lirc_t)) return(-EINVAL);
 	
 	add_wait_queue(&lirc_read_queue,&wait);
-	current->state=TASK_INTERRUPTIBLE;
+	set_current_state(TASK_INTERRUPTIBLE);
 	while(n<count)
 	{
 		if(rx_head!=rx_tail)
 		{
-			retval=verify_area(VERIFY_WRITE,
-					   (void *) buf+n,sizeof(lirc_t));
-			if (retval)
+			if(copy_to_user((void *) buf+n,
+					(void *) (rx_buf+rx_head),
+					sizeof(lirc_t)))
 			{
-				return retval;
+				retval=-EFAULT;
+				break;
 			}
-			copy_to_user((void *) buf+n,(void *) (rx_buf+rx_head),
-				     sizeof(lirc_t));
 			rx_head=(rx_head+1)&(RBUF_LEN-1);
 			n+=sizeof(lirc_t);
 		}
@@ -331,28 +336,21 @@ static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 				break;
 			}
 			schedule();
-			current->state=TASK_INTERRUPTIBLE;
+			set_current_state(TASK_INTERRUPTIBLE);
 		}
 	}
 	remove_wait_queue(&lirc_read_queue,&wait);
-	current->state=TASK_RUNNING;
+	set_current_state(TASK_RUNNING);
 	return (n ? n : retval);
 }
 static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t * pos)
 {
 	unsigned long flags;
-#ifdef LIRC_SIR_TEKRAM
-	return(-EBADF);
-#else
 	int i;
-	int retval;
 
         if(n%sizeof(lirc_t) || (n/sizeof(lirc_t)) > WBUF_LEN)
 		return(-EINVAL);
-	retval = verify_area(VERIFY_READ, buf, n);
-	if (retval)
-		return retval;
-	copy_from_user(tx_buf, buf, n);
+	if(copy_from_user(tx_buf, buf, n)) return -EFAULT;
 	i = 0;
 	n/=sizeof(lirc_t);
 #ifdef LIRC_ON_SA1100
@@ -383,7 +381,6 @@ static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t
 	Ser2UTCR3=UTCR3_RXE|UTCR3_RIE;
 #endif
 	return n;
-#endif
 }
 
 static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
@@ -393,16 +390,7 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 	unsigned long value = 0;
 #ifdef LIRC_ON_SA1100
 	unsigned int ivalue;
-#endif
 
-#ifdef LIRC_SIR_TEKRAM
-	if (cmd == LIRC_GET_FEATURES)
-		value = LIRC_CAN_REC_MODE2;
-	else if (cmd == LIRC_GET_SEND_MODE)
-		value = 0;
-	else if (cmd == LIRC_GET_REC_MODE)
-		value = LIRC_MODE_MODE2;
-#elif defined(LIRC_ON_SA1100)
 	if (cmd == LIRC_GET_FEATURES)
 		value = LIRC_CAN_SEND_PULSE |
 			LIRC_CAN_SET_SEND_DUTY_CYCLE |
@@ -466,15 +454,6 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 	
 	if (retval)
 		return retval;
-	
-#ifdef LIRC_SIR_TEKRAM
-	if (cmd == LIRC_SET_REC_MODE) {
-		if (value != LIRC_MODE_MODE2)
-			retval = -ENOSYS;
-	} else if (cmd == LIRC_SET_SEND_MODE) {
-		retval = -ENOSYS;
-	}
-#else
 	if (cmd == LIRC_SET_REC_MODE) {
 		if (value != LIRC_MODE_MODE2)
 			retval = -ENOSYS;
@@ -482,7 +461,7 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 		if (value != LIRC_MODE_PULSE)
 			retval = -ENOSYS;
 	}
-#endif
+
 	return retval;
 }
 
@@ -558,6 +537,7 @@ static struct lirc_plugin plugin = {
        set_use_inc:    set_use_inc,
        set_use_dec:    set_use_dec,
        fops:           &lirc_fops,
+       owner:          THIS_MODULE,
 };
 
 
@@ -664,9 +644,8 @@ static irqreturn_t sir_interrupt(int irq, void * dev_id,
 		deltv = delta(&last_tv, &curr_tv);
 		do
 		{
-			dprintk("t %lu , d %d\n", deltintrtv, (int)data);
 			data=Ser2UTDR;
-			//printk("data: %d\n",data);
+			dprintk("%d data: %u\n", n, (unsigned int) data);
 			n++;
 		}
 		while(status&UTSR0_RID && /* do not empty fifo in
@@ -813,7 +792,6 @@ static void send_space(unsigned long length)
 	off();
 	safe_udelay(length);
 }
-#elif defined(LIRC_SIR_TEKRAM)
 #else
 static void send_space(unsigned long len)
 {
@@ -825,10 +803,14 @@ static void send_pulse(unsigned long len)
 	long bytes_out = len / TIME_CONST;
 	long time_left;
 
-	if (!bytes_out)
-		bytes_out++;
 	time_left = (long)len - (long)bytes_out * (long)TIME_CONST;
-	while (--bytes_out) {
+	if (bytes_out == 0)
+	{
+		bytes_out++;
+		time_left = 0;
+	}
+	while (bytes_out--)
+       	{
 		outb(PULSE, io + UART_TX);
 		/* FIXME treba seriozne cakanie z drivers/char/serial.c */
 		while (!(inb(io + UART_LSR) & UART_LSR_THRE));
@@ -1006,6 +988,8 @@ static int init_hardware(void)
 	outb(UART_MCR_DTR|UART_MCR_RTS|UART_MCR_OUT2, io + UART_MCR);
 #ifdef LIRC_SIR_ACTISYS_ACT200L
 	init_act200();
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+	init_act220();
 #endif
 #endif
 	spin_unlock_irqrestore(&hardware_lock, flags);
@@ -1049,26 +1033,27 @@ static int init_port(void)
 {
 	int retval;
 	
-#ifndef LIRC_ON_SA1100
 	/* get I/O port access and IRQ line */
-	retval = check_region(io, 8);
-	if (retval < 0) {
+#ifndef LIRC_ON_SA1100
+	if(request_region(io, 8, LIRC_DRIVER_NAME) == NULL)
+	{
 		printk(KERN_ERR LIRC_DRIVER_NAME
-			": i/o port 0x%.4x already in use.\n",
-			io);
-		return retval;
+		       ": i/o port 0x%.4x already in use.\n", io);
+		return -EBUSY;
 	}
 #endif
 	retval = request_irq(irq, sir_interrupt, SA_INTERRUPT,
 			     LIRC_DRIVER_NAME, NULL);
 	if (retval < 0) {
+#               ifndef LIRC_ON_SA1100
+		release_region(io, 8);
+#               endif
 		printk(KERN_ERR LIRC_DRIVER_NAME
 			": IRQ %d already in use.\n",
 			irq);
 		return retval;
 	}
 #ifndef LIRC_ON_SA1100
-	request_region(io, 8, LIRC_DRIVER_NAME);
 	printk(KERN_INFO LIRC_DRIVER_NAME
 		": I/O port 0x%.4x, IRQ %d.\n",
 		io, irq);
@@ -1083,7 +1068,6 @@ static int init_port(void)
 
 static void drop_port(void)
 {
-	disable_irq(irq);
 	free_irq(irq, NULL);
 	del_timer_sync(&timerlist);
 #ifndef LIRC_ON_SA1100
@@ -1237,6 +1221,62 @@ static void init_act200(void)
 }
 #endif
 
+#ifdef LIRC_SIR_ACTISYS_ACT220L
+/* Derived from linux IrDA driver (drivers/net/irda/actisys.c) 
+ * Drop me a mail for any kind of comment: maxx@spaceboyz.net */
+
+void init_act220(void) {
+	int i;
+
+	/* DLAB 1 */
+	soutp(UART_LCR, UART_LCR_DLAB|UART_LCR_WLEN7);
+
+	/* 9600 baud */
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 12);
+
+	/* DLAB 0 */
+	soutp(UART_LCR, UART_LCR_WLEN7);
+
+	/* reset the dongle, set DTR low for 10us */
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_OUT2);
+	udelay(10);
+
+	/* back to normal (still 9600) */
+	soutp(UART_MCR, UART_MCR_DTR|UART_MCR_RTS|UART_MCR_OUT2);
+
+	/* send RTS pulses until we reach 115200
+	 * i hope this is really the same for act220l/act220l+ */
+	for(i = 0; i < 3; i++)
+	{
+		udelay(10);
+		/* set RTS low for 10 us */
+		soutp(UART_MCR, UART_MCR_DTR|UART_MCR_OUT2);
+		udelay(10);
+		/* set RTS high for 10 us */
+		soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
+	}
+	
+	/* back to normal operation */
+	udelay(1500); /* better safe than sorry ;) */
+
+	/* Set DLAB 1. */
+	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN7);
+
+	/* Set divisor to 1 => 115200 Baud */
+	soutp(UART_DLM,0);
+	soutp(UART_DLL,1);
+
+	/* Set DLAB 0, 7 Bit */
+	/* The dongle doesn't seem to have any problems with operation
+	   at 7N1 */
+	soutp(UART_LCR, UART_LCR_WLEN7);
+
+	/* enable interrupts */
+	soutp(UART_IER, UART_IER_RDI);
+}
+#endif
+
 static int init_lirc_sir(void)
 {
 	int retval;
@@ -1246,7 +1286,6 @@ static int init_lirc_sir(void)
 	if (retval < 0)
 		return retval;
 	init_hardware();
-	enable_irq(irq);
 	printk(KERN_INFO LIRC_DRIVER_NAME
 		": Installed.\n");
 	return 0;
@@ -1286,6 +1325,9 @@ MODULE_AUTHOR("Christoph Bartelmus");
 #elif defined(LIRC_SIR_ACTISYS_ACT200L)
 MODULE_DESCRIPTION("LIRC driver for Actisys Act200L");
 MODULE_AUTHOR("Karl Bongers");
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+MODULE_DESCRIPTION("LIRC driver for Actisys Act220L(+)");
+MODULE_AUTHOR("Jan Roemisch");
 #else
 MODULE_DESCRIPTION("Infrared receiver driver for SIR type serial ports");
 MODULE_AUTHOR("Milan Pikula");
