@@ -1,4 +1,4 @@
-/*      $Id: irpty.c,v 5.6 2001/11/14 18:05:52 ranty Exp $      */
+/*      $Id: irpty.c,v 5.9 2005/03/11 11:03:00 lirc Exp $      */
 
 /****************************************************************************
  ** irpty.c *****************************************************************
@@ -36,8 +36,16 @@
 #include <unistd.h>
 
 #ifdef HAVE_FORKPTY
+#ifdef __APPLE__
+#include <util.h>
+#else
+#if defined (__FreeBSD__)
+#include <libutil.h>
+#else
 #include <pty.h>
 #endif
+#endif
+#endif
 
 #include "lirc_client.h"
 
@@ -325,7 +333,7 @@ int main(int argc, char *argv[])
 			!= EOF) {
 		switch (c) {
 		case 'h':
-			printf("Usage: %s [options] [config_file] -- "
+			printf("Usage: %s [options] config_file -- "
 					"program [args ...]\n",argv[0]);
 			printf("\t -h --help \t\tdisplay usage summary\n");
 			printf("\t -V --version \t\tdisplay version\n");
